#version 430 core

uniform float zNear;
uniform float zFar;
uniform sampler2D depthMap0; // scene depth
//uniform sampler2D depthMap1; // cutsurface depth
uniform vec2 imgSize;
uniform float maxAlpha;

// Light information
uniform vec4 lightPosition;
uniform vec3 lightIntensity;

// Material information
uniform bool hasTexture;
uniform sampler2D Texture;
uniform vec3 Ka;
uniform vec3 Kd;
uniform vec3 Ks;
uniform float shininess;

// Input variables coming from vertex shader, interpolated to this fragment
in vec3 interpolatedPosition;
in vec3 interpolatedNormal;
in vec2 TexCoords;

out vec4 fragmentColor;

void main()
{
	
	float depth = -(interpolatedPosition.z + zNear) / (zFar - zNear); // -interpolatedPosition.z / zFar; // https://learnopengl.com/Advanced-OpenGL/Depth-testing
	vec2 texCoord = gl_FragCoord.xy / imgSize;
	float sceneDepth = texture(depthMap0, texCoord).r;
	//float cutsurfaceDepth = texture(depthMap1, texCoord).r;
	
	// This fragment was not cutted or is occluded by the scene, so we are not interested in it
	if (depth > sceneDepth)
	{
		discard;
	}
	
	//float alpha = clamp( maxAlpha - 2.0f * cutsurfaceDepth, 0.0, 1.0);

	vec3 ambient  = Ka;
	vec3 diffuse = Kd;
	vec3 specular = Ks;
	
	if (hasTexture)
	{
		vec3 color = vec3(texture(Texture, TexCoords));
		vec3 ambientIntensity = vec3(0.3f, 0.3f, 0.3f);
		vec3 diffuseIntensity = vec3(0.4f, 0.4f, 0.4f);
		vec3 specularIntensity = vec3(0.5f, 0.5f, 0.5f);
		ambient = ambientIntensity * color;
		diffuse = diffuseIntensity * color;
		specular = specularIntensity * color;
		
	}
	
	// normal has been interpolated, so we need to normalize it
	vec3 normalVector = normalize(interpolatedNormal);

	// Calculate light source vector
	vec3 lightSourceVector = normalize( lightPosition.xyz - interpolatedPosition);

	// Calculate the view vector
	vec3 viewVector = normalize( -interpolatedPosition.xyz );

	// Ambient contribution
	vec3 ambientContribution = ambient;

	// Default black diffuse and specular contributions
	vec3 diffuseContribution = vec3(0.0);
	vec3 specularContribution = vec3(0.0);

	if( dot( lightSourceVector, normalVector ) > 0.0 ) {

		// Diffuse Contribution
		diffuseContribution = diffuse * dot( lightSourceVector, normalVector);

		// halfway vector, reduces the need for calculating a reflected vector,
		// which improvies performance slightly
		vec3 halfwayVector = normalize( viewVector + lightSourceVector);

		// Specular contribution
		specularContribution = specular * pow( dot(halfwayVector, normalVector), shininess);
	}

	// Calculate final color
	fragmentColor = vec4(lightIntensity * (ambientContribution + diffuseContribution + specularContribution), maxAlpha);
	
    

}