#version 430 core

#define M_PI 3.14159265358979323846

uniform bool edgeCompression;
uniform bool directionalConstraint;

uniform vec2 imgSize;
uniform float theta;
uniform float zNear;
uniform float zFar;

in vec3 viewPosition;

out vec4 outColor;

void main(void)
{
	// Current depth (linearized) https://learnopengl.com/Advanced-OpenGL/Depth-testing
	float z = -(viewPosition.z + zNear) / (zFar - zNear);

	// Use theta as angle per default
	float thetaEdge = theta;
	float thetaDirectional = theta;

	// If edge compression is on set thetaEdge
	if (edgeCompression)
	{
		float r = 0.1f; // screen edge region as in paper
		vec2 ndc = (gl_FragCoord.xy / imgSize.xy) * 2.0f - 1.0f; // NDC [-1 1]
		float edgeX = clamp( (1.0f - abs(ndc.x)) / r, 0.0f, 1.0f);
		float edgeY = clamp( (1.0f - abs(ndc.y)) / r, 0.0f, 1.0f);
		thetaEdge = theta * edgeX * edgeY;
				
		// If directional constraint is on set thetaDirectional
		if (directionalConstraint)
		{
			float phi = (M_PI / 2.0f) - acos( dot( normalize( -viewPosition ), vec3(0.0f, 1.0f, 0.0f) ) );
			thetaDirectional = clamp(thetaEdge, 0.0, abs(phi)) + thetaEdge * smoothstep(0.0f, (M_PI / 18.0f), -phi); // https://www.khronos.org/registry/OpenGL-Refpages/gl4/html/smoothstep.xhtml
		}
		else
		{
			thetaDirectional = thetaEdge;
		}
	}
	
	// Normalize thetas (x - min) / (max - min) -> [0 1]
	thetaEdge = thetaEdge / theta;
	thetaDirectional = thetaDirectional / theta;
	
	outColor = vec4(gl_FragCoord.x, gl_FragCoord.y, thetaEdge, thetaDirectional); // save screen position and angles in colorMap
	gl_FragDepth = z;
	
}